package com.android.launcher3.shortcuts.sesame


import android.content.ComponentName
import android.content.ContentResolver
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Resources
import android.graphics.drawable.Drawable
import android.net.Uri
import android.os.UserHandle
import android.support.v4.content.ContextCompat
import android.support.v4.content.res.ResourcesCompat
import android.text.Html
import android.text.TextUtils
import android.util.Log
import com.android.launcher3.compat.UserHandleCompat
import com.android.launcher3.compat.UserManagerCompat
import com.android.launcher3.config.ProviderConfig
import com.android.launcher3.shortcuts.DeepShortcutInfoCompat
import com.android.launcher3.shortcuts.DeepShortcutInfoCompatBackport
import com.teslacoilsw.kotlin.getNumberExtraAsLong
import com.teslacoilsw.launcher.BuildConfig
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.deepshortcutproxy.DSPDispatcherActivity
import com.teslacoilsw.launcher.util.PackageNames
import org.json.JSONObject
import java.io.FileNotFoundException
import java.net.URISyntaxException
import java.util.*

private const val TAG = "SesameLink"
private val GENERIC_ICON_URI = Uri.parse("android.resource://" + BuildConfig.APPLICATION_ID + "/drawable/launcher_shortcut_generic")

fun getShortcuts(cr: ContentResolver, cn: ComponentName, user: UserHandleCompat): List<DeepShortcutInfoCompat>? {

    val c = try {
        cr.query(SesameLinkContract.LINK_CONTENT_URI_LIMIT_5, null, null, arrayOf(cn.packageName), null)
    } catch (e: Exception) {
        null
    }
    if (c == null) {
        Log.e(TAG, "ERROR: failed to create cursor for " + SesameLinkContract.LINK_CONTENT_URI_LIMIT_5)
        return null
    }

    // read in the data
    val iconIdx = c.getColumnIndex(SesameLinkContract.COL_ICON)
    val labelIdx = c.getColumnIndex(SesameLinkContract.COL_DISPLAY_LABEL)
//    val pkgIdx = c.getColumnIndex(SesameLinkContract.COL_PACKAGE)
    val intentIdx = c.getColumnIndex(SesameLinkContract.COL_INTENT)
    val typeIdx = c.getColumnIndex(SesameLinkContract.COL_TYPE)
    val typeDataIdx = c.getColumnIndex(SesameLinkContract.COL_TYPE_DATA)
//    val directIntentIdx = c.getColumnIndex(SesameLinkContract.COL_DIRECT_INTENT)
    val shortcuts = ArrayList<DeepShortcutInfoCompat>(c.count)
    var i = 0
    while (c.moveToNext()) {
        val iconUriStr = c.getString(iconIdx)
        val label = c.getString(labelIdx)
//        val pkgName = c.getString(pkgIdx)
        val intentUri = c.getString(intentIdx)
        val type = c.getString(typeIdx)
//        val directIntentUri = c.getString(directIntentIdx)
        val typeData = c.getString(typeDataIdx)

        val dsp = DeepShortcutInfoCompatBackport(cn, label, null, null, Integer.toString(i), i, user, true, 0)

        try {
            dsp.intents = arrayOf(Intent.parseUri(intentUri, 0))
            shortcuts.add(dsp)

            val iconUri: Uri = if (TextUtils.isEmpty(iconUriStr)) {
                GENERIC_ICON_URI
            } else {
                Uri.parse(iconUriStr)
            }
            dsp.iconUri = iconUri
            dsp.shouldClipIconToCircle = type == SesameLinkContract.TYPE_CONTACT
        } catch (e: URISyntaxException) {
            e.printStackTrace()
        }

        i++

    }
    c.close()

    return shortcuts
}

fun deoptimizeSesame(context: Context): Boolean {
    return try {
        val intent = Intent("com.teslacoilsw.BLANK_ACTIVITY");
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS)
        intent.setPackage(PackageNames.SESAME)
        context.startActivity(intent)
        true
    } catch (e: Exception) {
        false
    }
}

fun search(context: Context, query: String, maxCount: Int): List<SesameShortcut> {
    val uri = SesameLinkContract.buildQueryUri(SesameLinkContract.SEARCH_CONTENT_URI, maxCount, SesameLinkContract.TYPE_CONTACT, SesameLinkContract.TYPE_SHORTCUT)
    var c = try {
        context.contentResolver.query(uri, null, null, arrayOf(query), null)
    } catch (e: Exception) {
        null
    }

    if (c == null) {
        /* This could be due to OEM "optimizations", try to bypass */
        if (deoptimizeSesame(context))
            c = context.contentResolver.query(uri, null, null, arrayOf(query), null)

        if (c == null) {
            Log.e(TAG, "ERROR: failed to create cursor for " + SesameLinkContract.SEARCH_CONTENT_URI_LIMIT_15)
            return emptyList()
        }
    }

    // read in the data
    val iconIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_ICON)
    val labelIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_DISPLAY_LABEL)
    val searchResultLabelIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_SEARCH_RESULT_LABEL)
//    val pkgIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_PACKAGE)
    val intentIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_INTENT)
    val directIntentIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_DIRECT_INTENT)
    val componentIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_COMPONENT)
    val typeIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_TYPE)
    val typeDataIdx = c.getColumnIndexOrThrow(SesameLinkContract.COL_TYPE_DATA)

    val shortcuts = ArrayList<SesameShortcut>(c.count)
    while (c.moveToNext()) {
        var iconUri = c.getString(iconIdx)?.let { Uri.parse(it) }
        val label = c.getString(labelIdx) ?: ""
        val searchResultLabel = c.getString(searchResultLabelIdx)?.let { Html.fromHtml(it) } ?: label
        val proxyIntent: Intent = c.getString(intentIdx)?.let {
            try { Intent.parseUri(it, 0) }
            catch (e: URISyntaxException) { e.printStackTrace(); null }
        } ?: continue
        val directIntent: Intent = c.getString(directIntentIdx)?.let {
            try { Intent.parseUri(it, 0) }
            catch (e: URISyntaxException) { e.printStackTrace(); null }
        } ?: proxyIntent
        val component = c.getString(componentIdx)?.let { ComponentName.unflattenFromString(it) }
        val type = c.getString(typeIdx) ?: continue
        val typeData = c.getString(typeDataIdx)?.let { try { JSONObject(it) } catch (e: Exception) { null } } ?: JSONObject();

        if (iconUri == null && DSPDispatcherActivity.COMPONENT_NAME == directIntent.component) {
            val user: UserHandle? = directIntent.getParcelableExtra("userHandle")
            val userId = user?.let { UserManagerCompat.getInstance(context).getSerialNumberForUser(UserHandleCompat.fromUser(it)) }
                    ?: directIntent.getNumberExtraAsLong("userId", 0)
            iconUri = Uri.Builder().scheme(ContentResolver.SCHEME_CONTENT).authority(ProviderConfig.DEEPSHORTCUT_PROXY_AUTHORITY).appendPath("icon").appendPath(userId.toString()).appendPath(directIntent.getStringExtra("packageName")).appendPath(directIntent.getStringExtra("id")).build()
        }
        val s = SesameShortcut(component = component,
                               displayLabel = label,
                               resultLabel = searchResultLabel,
                               iconUri = iconUri,
                               proxyIntent = proxyIntent,
                               directIntent = directIntent,
                               type = type,
                               typeData =  typeData)
        shortcuts.add(s)
    }
    c.close()

    return shortcuts
}

private fun getDrawableFromUri(mContext: Context, uri: Uri): Drawable? {
    val scheme = uri.scheme
    if (ContentResolver.SCHEME_ANDROID_RESOURCE == scheme) {
        try {
            val result = getResourceId(mContext, uri)
            return ResourcesCompat.getDrawable(result.r, result.id, mContext.theme) ?:
                    ContextCompat.getDrawable(mContext, R.drawable.launcher_shortcut_generic)
        } catch (e: Exception) {
            Log.w(TAG, "Unable to open content: " + uri, e)
        }

    } else if (ContentResolver.SCHEME_CONTENT == scheme || ContentResolver.SCHEME_FILE == scheme) {
        try {
            mContext.contentResolver.openInputStream(uri)?.use {
                return Drawable.createFromResourceStream(mContext.resources, null, it, null)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    } else {
        return Drawable.createFromPath(uri.toString())
    }

    return null
}

data class SesameShortcut(var component: ComponentName?,
                          val displayLabel: String,
                          val resultLabel: CharSequence,
                          val iconUri: Uri?,
                          val proxyIntent: Intent,
                          val directIntent: Intent,
                          val type: String,
                          val typeData: JSONObject,
                          val user: UserHandleCompat = UserHandleCompat.myUserHandle())

@Throws(FileNotFoundException::class)
private fun getResourceId(context: Context, uri: Uri): OpenResourceIdResult {
    val authority = uri.authority
    val r: Resources
    if (TextUtils.isEmpty(authority)) {
        throw FileNotFoundException("No authority: " + uri)
    } else {
        try {
            r = context.packageManager.getResourcesForApplication(authority)
        } catch (ex: PackageManager.NameNotFoundException) {
            throw FileNotFoundException("No package found for authority: " + uri)
        }

    }
    val path = uri.pathSegments ?: throw FileNotFoundException("No path: " + uri)
    val len = path.size
    val id: Int
    if (len == 1) {
        try {
            id = Integer.parseInt(path[0])
        } catch (e: NumberFormatException) {
            throw FileNotFoundException("Single path segment is not a resource ID: " + uri)
        }

    } else if (len == 2) {
        id = r.getIdentifier(path[1], path[0], authority)
    } else {
        throw FileNotFoundException("More than two path segments: " + uri)
    }
    if (id == 0) {
        throw FileNotFoundException("No resource found for: " + uri)
    }
    val res = OpenResourceIdResult(r, id)
    return res
}

private data class OpenResourceIdResult(val r: Resources, val id: Int)
