@file:Suppress("LoopToCallChain")

package com.teslacoilsw.kotlin

import android.support.v4.util.SimpleArrayMap

/**
 * A handful of extension functions to make working with specific types of Collections easier
 *
 * Some of these are essentially replacements for the stdlibs methods, but avoid the
 * creation of an iterator. These are prefixed with "q" (for "quick" or whatever) to avoid
 * competing with the stdlib's methods.
 */


/** List
 * Not that these methods could be very inefficient if used on the wrong type of list
 * Possible a LinkedList
 * In practice this is unlikely to be an issue
 **/
inline fun <T> List<T>.qForEach(action: (T) -> Unit) {
    for (i in indices)
        action.invoke(get(i))
}

inline fun <T> List<T>.qForEachWithIndex(action: (Int, T) -> Unit) {
    for (i in indices)
        action.invoke(i, get(i))
}


inline fun <T> List<T>.qFind(predicate: (T) -> Boolean): T? {
    for (i in indices) {
        val v = get(i)
        if (predicate(v))
            return v
    }
    return null
}

/** SimpleArrayMap/ArrayMap **/
inline fun <K, V> SimpleArrayMap<K, V>.qForEach(action: (K, V) -> Unit) {
    for (i in 0..size()-1)
        action.invoke(keyAt(i), valueAt(i))
}

inline fun <K, V> SimpleArrayMap<K, V>.qForEachWithIndex(action: (Int, K, V) -> Unit) {
    for (i in 0..size()-1)
        action.invoke(i, keyAt(i), valueAt(i))
}

@Suppress("NOTHING_TO_INLINE")
inline operator fun  <K, V> SimpleArrayMap<K, V>.set(k: K, value: V) {
    put(k, value)
}
