package com.teslacoilsw.launcher

import android.app.Activity
import android.content.Intent
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.os.Bundle
import com.afollestad.materialdialogs.MaterialDialog
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.preferences.SettingsActivity
import com.teslacoilsw.launcher.util.PackageNames
import timber.log.Timber

private val EXTRA_FIRST_RUN = "first_run"
private val TAG = "Nova.Integrations"
class IntegrationPermissionActivity : Activity() {
    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setResult(Activity.RESULT_CANCELED)

        if (intent == null || callingActivity == null) {
            Timber.tag(TAG).e("Could not verify callers identity, did you forget to startActivityForResult?")
            finish()
            return
        }

        var integrationPackage = callingActivity.packageName
        if (integrationPackage != PackageNames.SESAME) {
            Timber.tag(TAG).e("Unknown integration request from package $integrationPackage")
            finish()
            return
        }

        val pm = applicationContext.packageManager
        val pi: PackageInfo
        try {
            pi = pm.getPackageInfo(integrationPackage, 0)
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
            finish()
            return
        }

        val integrationName = pi.applicationInfo.loadLabel(pm).toString()

        val sesame_pref_app_search = Pref2.get().sesame_app_search
        val sesame_pref_menu = Pref2.get().sesame_shortcut_menu

        val firstRun = intent.getBooleanExtra(EXTRA_FIRST_RUN, false)
        if (firstRun && (!sesame_pref_app_search.isSet || sesame_pref_app_search.get()!!)) {
            /* We essentially default allow this the first time, but if the user turns it off
             * then Sesame will need to go through the full prompt */
            sesame_pref_app_search.set(true)
            sesame_pref_menu.set(true)
            startNovaAfterIntegration(integrationPackage)
            setResult(RESULT_OK)
            finish()
            return
        }

        if (sesame_pref_app_search.get()!! && sesame_pref_menu.get()!!) {
            setResult(RESULT_OK)
            finish()
        } else {
            val d = MaterialDialog.Builder(this)
                    .title(R.string.allow_integration)
                    .iconRes(R.mipmap.ic_launcher_home)
                    .content(getString(R.string.allow_integration_long, integrationName))
                    .negativeText(R.string.cancel)
                    .positiveText(R.string.allow)
                    .onPositive { _, _ ->
                        sesame_pref_app_search.set(true)
                        sesame_pref_menu.set(true)
                        setResult(RESULT_OK)
                        if (firstRun)
                            startNovaAfterIntegration(integrationPackage)
                    }.build()
            d.setOnDismissListener {
                finish()
            }
            d.show()
        }
    }

    private fun startNovaAfterIntegration(pkg: String) {
//        if (NovaAction.valueOf(Pref.Value.home_button_intent) != NovaAction.APP_SEARCH) {
            val intent = Intent("com.teslacoilsw.launcher.CONFIGURE_INTEGRATION")
            intent.component = SettingsActivity.COMPONENT_NAME
            intent.putExtra("package", pkg)
            startActivity(intent)
//        } else {
//            val intent = NovaAction.APP_SEARCH.intent
//            startActivity(intent)
//        }
    }
}
