package com.teslacoilsw.launcher.bitmaputils

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Matrix
import android.graphics.Paint
import android.graphics.Path
import android.graphics.Rect
import android.graphics.RectF

class ThreadLocalBitmapOptions : ThreadLocal<BitmapFactory.Options>() {

    override fun initialValue(): BitmapFactory.Options {
        val opts = BitmapFactory.Options()
        opts.inTempStorage = BitmapTempStorage.acquire()
        return opts
    }

    override fun get(): BitmapFactory.Options {
        return super.get().apply {
            inPreferredConfig = Bitmap.Config.ARGB_8888
            inJustDecodeBounds = false
        }
    }
}

class ThreadLocalCanvas : ThreadLocal<Canvas>() {
    override fun initialValue(): Canvas {
        return Canvas()
    }

    fun withBitmap(bmp: Bitmap, body: Canvas.() -> Unit) {
        val c = get()
        try {
            c.setBitmap(bmp)
            body(c)
        } finally {
            c.setBitmap(null)
        }
    }
}

class ThreadLocalPath : ThreadLocal<Path>() {
    override fun initialValue(): Path {
        return Path()
    }

    override fun get(): Path {
        return super.get().apply { reset() }
    }
}

class ThreadLocalFloatArray2 : ThreadLocal<FloatArray>() {
    override fun initialValue(): FloatArray {
        return FloatArray(2)
    }
}

class ThreadLocalRect : ThreadLocal<Rect>() {
    override fun initialValue(): Rect {
        return Rect()
    }

    fun get(l: Int,  t: Int, r: Int, b: Int): Rect = get().apply { set(l, t, r, b) }
    fun get(r: Rect): Rect = get().apply { set(r) }
}

class ThreadLocalRectF : ThreadLocal<RectF>() {
    override fun initialValue(): RectF {
        return RectF()
    }

    fun get(l: Float,  t: Float, r: Float, b: Float): RectF {
        val rect = get()
        rect.set(l, t, r, b)
        return rect
    }
}

class ThreadLocalMatrix: ThreadLocal<Matrix>() {
    override fun initialValue(): Matrix {
        return Matrix()
    }

    override fun get(): Matrix {
        return super.get().apply { reset() }
    }
}

class ThreadLocalPaint: ThreadLocal<Paint>() {
    override fun initialValue(): Paint {
        return Paint(Paint.ANTI_ALIAS_FLAG or Paint.DITHER_FLAG or Paint.FILTER_BITMAP_FLAG)
    }

    override fun get(): Paint {
        return super.get().apply {
            color = Color.WHITE
            shader = null
            style = Paint.Style.FILL
            xfermode = null
        }
    }
}