package com.teslacoilsw.launcher.deepshortcutproxy

import android.app.Activity
import android.content.ComponentName
import android.content.Context
import android.content.pm.LauncherApps
import android.os.Build
import android.os.Bundle
import android.os.UserHandle
import android.support.annotation.RequiresApi
import android.text.TextUtils
import android.widget.Toast
import com.android.launcher3.compat.UserManagerCompat
import com.teslacoilsw.kotlin.getNumberExtraAsLong
import com.teslacoilsw.launcher.BuildConfig
import com.teslacoilsw.launcher.util.PackageNames
import timber.log.Timber

val COMPONENT_DSPDispatcherActivity = ComponentName(BuildConfig.APPLICATION_ID, DSPDispatcherActivity::class.java.name)
@RequiresApi(api = Build.VERSION_CODES.N_MR1)
class DSPDispatcherActivity : Activity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        if (intent == null || TextUtils.isEmpty(callingPackage)) {
            Timber.tag(TAG).e("Could not verify callers identity, did you forget to startActivityForResult?")
            setResult(Activity.RESULT_CANCELED)
            finish()
            return
        }

        if (!PackageNames.SESAME.equals(callingPackage)) {
            Timber.tag(TAG).e("Rejecting unauthorized package $callingPackage from DeepShortcutProxyDispatcherActivity")
            setResult(Activity.RESULT_CANCELED)
            finish()
            return
        }

        val apps = getSystemService(Context.LAUNCHER_APPS_SERVICE) as LauncherApps

        val intent = intent
        val packageName = intent.getStringExtra("packageName")
        val id = intent.getStringExtra("id")
        val sourceBounds = intent.sourceBounds
        val activityOptions = intent.getBundleExtra("activityOptions")
        val user = intent.getParcelableExtra<UserHandle>("userHandle")
                ?: UserManagerCompat.getInstance(this)
                    .getUserForSerialNumber(intent.getNumberExtraAsLong("userId", 0)).user

        try {
            apps.startShortcut(packageName, id, sourceBounds, activityOptions, user)
            setResult(Activity.RESULT_OK)
        } catch (ex: Exception) {
            Toast.makeText(this, ex.localizedMessage, Toast.LENGTH_SHORT).show()
            Timber.e(ex, "Unable to start shortcut ${intent.toUri(0)} $packageName $id $user")
            setResult(Activity.RESULT_CANCELED)
        }

        finish()
    }

    companion object {
        val COMPONENT_NAME = ComponentName(BuildConfig.APPLICATION_ID, DSPDispatcherActivity::class.java.name)
        private val TAG = "Nova.DSP"
    }
}
