package com.teslacoilsw.launcher.deepshortcutproxy

import android.content.pm.LauncherApps
import android.content.pm.ShortcutInfo
import android.os.Build
import android.os.ParcelFileDescriptor
import android.os.UserHandle
import android.support.annotation.RequiresApi
import com.teslacoilsw.shared.util.ReflectionHelper

private val _hasIconResource = ReflectionHelper.findPrivateMethod(ShortcutInfo::class.java, "hasIconResource")
fun ShortcutInfo.reflectHasIconResource(): Boolean {
    _hasIconResource?.let {
        try {
            val out = _hasIconResource.invoke(this)
            if (out is Boolean)
                return out
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return false
}

private val _hasIconFile = ReflectionHelper.findPrivateMethod(ShortcutInfo::class.java, "hasIconFile")
fun ShortcutInfo.reflectHasIconFile(): Boolean {
    _hasIconFile?.let {
        try {
            val out = _hasIconFile.invoke(this)
            if (out is Boolean)
                return out
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return false
}

private val _getShortcutIconFd = ReflectionHelper.findPrivateMethod(LauncherApps::class.java,
        "getShortcutIconFd", String::class.java, String::class.java, UserHandle::class.java)
@RequiresApi(Build.VERSION_CODES.N_MR1)
fun LauncherApps.reflectionGetShortcutIconFd(shortcutInfo: ShortcutInfo): ParcelFileDescriptor? {
    _getShortcutIconFd.let {
        try {
            return _getShortcutIconFd.invoke(this, shortcutInfo.`package`, shortcutInfo.id, shortcutInfo.userHandle) as ParcelFileDescriptor
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return null
}

private val _getIconResourceId = ReflectionHelper.findPrivateMethod(ShortcutInfo::class.java, "getIconResourceId")
fun ShortcutInfo.reflectGetIconResourceId(): Int {
    _getIconResourceId?.let {
        try {
            return it.invoke(this) as Int
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return 0
}