package com.teslacoilsw.launcher.icon

import android.content.res.ColorStateList
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.ColorFilter
import android.graphics.Matrix
import android.graphics.Paint
import android.graphics.Path
import android.graphics.PixelFormat
import android.graphics.Rect
import android.graphics.drawable.Drawable

class PathDrawable(val path: Path, val width: Int, val height: Int): Drawable() {

    private var color: ColorStateList = ColorStateList.valueOf(Color.WHITE)
    private val tempMatrix = Matrix()
    private var localPath = Path()
    private val paint = Paint(Paint.ANTI_ALIAS_FLAG or Paint.DITHER_FLAG)

    override fun onBoundsChange(bounds: Rect) {
        super.onBoundsChange(bounds)
        tempMatrix.setScale(bounds.width()/width.toFloat(), bounds.height()/height.toFloat())

        path.transform(tempMatrix, localPath)
    }

    override fun draw(canvas: Canvas) {
        paint.color = color.getColorForState(state, color.defaultColor)
        canvas.drawPath(localPath, paint)
    }

    fun setColor(color: Int) {
        setColor(ColorStateList.valueOf(color))
    }

    fun setColor(_color: ColorStateList) {
        color = _color
    }

    override fun isStateful(): Boolean {
        return color.isStateful
    }

    override fun getIntrinsicWidth() = width
    override fun getIntrinsicHeight() = height

    override fun setAlpha(alpha: Int) {
        paint.alpha = alpha
    }

    override fun getOpacity() = PixelFormat.TRANSLUCENT

    override fun setColorFilter(colorFilter: ColorFilter?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

}
