package com.teslacoilsw.launcher.preferences

 import android.graphics.Color
 import android.graphics.drawable.BitmapDrawable
 import android.graphics.drawable.ColorDrawable
 import android.graphics.drawable.Drawable
 import android.os.Build
 import android.os.Bundle
 import android.support.v4.content.ContextCompat
 import android.util.SparseArray
 import android.view.View
 import android.widget.ImageView
 import android.widget.LinearLayout
 import android.widget.RadioButton
 import android.widget.RadioGroup
 import aosp.android.graphics.drawable.InsetDrawableProportional
 import com.android.launcher3.BubbleTextView
 import com.android.launcher3.LauncherAppState
 import com.android.launcher3.LauncherModel
 import com.android.launcher3.Utilities
 import com.teslacoilsw.kotlin.children
 import com.teslacoilsw.launcher.BuildConfig
 import com.teslacoilsw.launcher.R
 import com.teslacoilsw.launcher.icon.AdaptiveIconShape
 import com.teslacoilsw.launcher.icon.NovaAdaptiveIconDrawable
 import com.teslacoilsw.launcher.icon.PathDrawable
 import com.teslacoilsw.launcher.icon.PathParser
 import com.teslacoilsw.launcher.util.DpUtil
 import com.teslacoilsw.launcher.util.palette.ColorUtils
 import com.teslacoilsw.launcher.util.palette.Palette
 import com.teslacoilsw.launcher.util.palette.PaletteHelper
 import com.teslacoilsw.matcompat.TintableSwitchCompat
 import com.teslacoilsw.shared.colorpicker.ColorUtil
 import com.teslacoilsw.shared.poisonlollipop.activity.PoisonActionBarActivity
 import kotterknife.bindView
 import timber.log.Timber

class AdaptiveIconSettingsActivity: PoisonActionBarActivity() {

    private val actionBarBack: ImageView by bindView(R.id.back)

    private val shapeGroup: RadioGroup by bindView(R.id.shape_group)
    private val shapeDefault: RadioButton by bindView(R.id.shape_default)
    private val shapeCircle: RadioButton by bindView(R.id.shape_circle)
    private val shapeSquircle: RadioButton by bindView(R.id.shape_squircle)
    private val shapeRoundedSquare: RadioButton by bindView(R.id.shape_rounded_square)
    private val shapeSquare: RadioButton by bindView(R.id.shape_square)
    private val shapeTeardrop: RadioButton by bindView(R.id.shape_teardrop)

    private val adaptiveIcons = SparseArray<NovaAdaptiveIconDrawable>()
    private val legacyIcon: BubbleTextView by bindView(R.id.legacy_icon)
    private val maskLegacySwitch: TintableSwitchCompat by bindView(R.id.mask_legacy_switch)
    private val preferLegacySwitch: TintableSwitchCompat by bindView(R.id.prefer_legacy_switch)
    private val adaptiveAnimSwitch: TintableSwitchCompat by bindView(R.id.adaptive_anim_switch)
    private val iconSize = DpUtil.getPixelSize(64)

    private val shapeButtons by lazy(LazyThreadSafetyMode.NONE) { arrayOf(shapeDefault, shapeCircle, shapeSquircle, shapeRoundedSquare, shapeSquare, shapeTeardrop) }
    private lateinit var currentShape: AdaptiveIconShape
    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        NovaAdaptiveIconDrawable.clearMask()

        overridePendingTransition(0, 0)
        setContentView(R.layout.activity_adaptiveicon)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Utilities.setStatusAndNavTransparent(window, true, false, false)
            window.statusBarColor = 0x3d000000
        }

        actionBarBack.setOnClickListener({
            onBackPressed()
        })

        currentShape = Pref2.get().adaptive_icon_shape.get()!!

        shapeDefault.tag = AdaptiveIconShape.DEFAULT
        shapeCircle.tag = AdaptiveIconShape.CIRCLE
        shapeSquircle.tag = AdaptiveIconShape.SQUIRCLE
        shapeRoundedSquare.tag = AdaptiveIconShape.ROUNDED_SQUARE
        shapeSquare.tag = AdaptiveIconShape.SQUARE
        shapeTeardrop.tag = AdaptiveIconShape.TEARDROP

        if (BuildConfig.DEBUG)
            Timber.v("System Path ${AdaptiveIconShape.DEFAULT.pathString}")
        for (shapeButton in shapeButtons) {
            val shape = shapeButton.tag as AdaptiveIconShape

            shapeButton.setCompoundDrawables(null, createDrawable(shape), null, null)
            if (shape != AdaptiveIconShape.DEFAULT && shape.pathEquals(AdaptiveIconShape.DEFAULT.pathString)) {
                shapeButton.visibility = View.GONE
                shapeDefault.text = shapeButton.text.toString() + "\n(Default)"
            }
            if (shape == Pref2.get().adaptive_icon_shape.get())
                shapeButton.isChecked = true
        }

        shapeGroup.setOnCheckedChangeListener { group, checkedId ->
            group.children().forEach {
                if (it.id != checkedId && it is RadioButton) {
                    it.isChecked = false
                    /* Workaround for 5.0/Asus items staying highlighted. TODO fix text color */
                    it.invalidate()
                }
            }
            currentShape = findViewById<RadioButton>(checkedId).tag as AdaptiveIconShape
            for (i in 0..adaptiveIcons.size()-1) {
                val icon = adaptiveIcons[i]
                icon.setShape(currentShape)
            }
        }

        maskLegacySwitch.setFinalOnCheckedChangeListener { _, newValue ->
            if (newValue && preferLegacySwitch.isChecked)
                preferLegacySwitch.isChecked = false
            updateLegacyIcon()
        }
        maskLegacySwitch.isChecked = Pref2.get().adaptive_icon_plate.get() ?: false
        findViewById<View>(R.id.plate_legacy_frame).setOnClickListener { maskLegacySwitch.toggle() }

        preferLegacySwitch.isChecked = Pref2.get().adaptive_icon_prefer_legacy.get() ?: false
        preferLegacySwitch.setFinalOnCheckedChangeListener { _, newValue ->
            if (newValue && maskLegacySwitch.isChecked)
                maskLegacySwitch.isChecked = false
            updateAllIcons()
        }

        adaptiveAnimSwitch.isChecked = Pref2.get().adaptive_icon_anim.get()!!
        findViewById<View>(R.id.adaptive_anim_frame).setOnClickListener { adaptiveAnimSwitch.toggle() }

        findViewById<View>(R.id.prefer_legacy_frame).setOnClickListener { preferLegacySwitch.toggle() }

        findViewById<View>(R.id.labs).visibility =
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O &&
                (Pref.Value.mPrefs.getBoolean(Pref.Key.EXPERIMENTAL_MODE, false) ||
                Pref2.get().adaptive_icon_backport.get()!!))
                View.VISIBLE
            else
                View.GONE

        updateAllIcons()
    }

    private fun updateAllIcons() {
        val x: LinearLayout = findViewById(R.id.preview_icons)
        x.children().forEachIndexed { i, v ->
            if (v is BubbleTextView) {
                if (v.id == R.id.legacy_icon) {
                    updateLegacyIcon()
                } else {
                    if (preferLegacySwitch.isChecked && i != 0) {
                        val resId = when (i) {
                            1 -> R.drawable.ic_lptheme_settings
                            else -> R.drawable.ic_lptheme_calculator
                        }
                        val d = ContextCompat.getDrawable(this, resId)!!
                        d.setBounds(0, 0, iconSize, iconSize)
                        v.applyCompoundDrawables(d);
                    } else {
                        val d: NovaAdaptiveIconDrawable
                        if (adaptiveIcons[i] != null) {
                            d = adaptiveIcons[i]
                        } else {
                            val resId = when (i) {
                                0 -> R.drawable.ic_launcher_default_adaptive
                                1 -> R.drawable.adaptive_preview_settings
                                else -> R.drawable.adaptive_preview_calculator
                            }
                            d = NovaAdaptiveIconDrawable.create(resources, resId, null)!!
                            d.setShape(currentShape)
                        }
                        d.setBounds(0, 0, iconSize, iconSize)
                        v.applyCompoundDrawables(d);
                        adaptiveIcons.put(i, d)
                    }
                }
            }
        }
    }

    private fun updateLegacyIcon() {
        var d = ContextCompat.getDrawable(this, R.mipmap.ic_launcher_default)!!
        if (maskLegacySwitch.isChecked) {
            if (adaptiveIcons[3] == null) {
                var tintColor = Color.WHITE
                val palette = Palette.generate((d as BitmapDrawable).bitmap)
                tintColor = PaletteHelper.getBestHighlightColorFromIconPalette(palette, false)
                tintColor = if (ColorUtil.isGray(tintColor))
                    Color.WHITE
                else
                    ColorUtils.getMutedColor(tintColor, 0.54f)

                d = NovaAdaptiveIconDrawable(ColorDrawable(tintColor),
                        InsetDrawableProportional(d, .25f))
                d.setShape(currentShape)
                adaptiveIcons.put(3, d)
            }
            d = adaptiveIcons[3]
        }
        d.setBounds(0, 0, iconSize, iconSize)
        legacyIcon.applyCompoundDrawables(d);
    }

    private fun createDrawable(shape: AdaptiveIconShape): Drawable {
        val d = PathDrawable(PathParser.createPathFromPathData(shape.pathString), 100, 100)
        d.setColor(ContextCompat.getColorStateList(this, R.color.btn_radio_drawable_color)!!)
        val dp48 = DpUtil.getPixelSize(48)
        d.setBounds(0, 0, dp48, dp48)
        return d
    }

    override fun onPause() {
        super.onPause()
        var needsRefresh = false
        for (shapeButton in shapeButtons) {
            if (shapeButton.isChecked) {
                val shape = shapeButton.tag as AdaptiveIconShape
                if (Pref2.get().adaptive_icon_shape.get() != shape) {
                    Pref2.get().adaptive_icon_shape.set(shape)
                    needsRefresh = true
                }
                break
            }
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val prefPreferLegacy = Pref2.get().adaptive_icon_prefer_legacy
            val preferLegacy = preferLegacySwitch.isChecked
            if (prefPreferLegacy.get() != preferLegacy) {
                prefPreferLegacy.set(preferLegacy)
                needsRefresh = true
            }
        }

        val prefMask = Pref2.get().adaptive_icon_plate
        if (prefMask.get() != maskLegacySwitch.isChecked) {
            prefMask.set(maskLegacySwitch.isChecked)
            needsRefresh = true
        }

        Pref2.get().adaptive_icon_anim.set(adaptiveAnimSwitch.isChecked)

        if (needsRefresh) {
            LauncherAppState.getInstance().iconCache.flushCustomIcons()
            LauncherAppState.getInstance().iconCache.setIconThemeFromConfig()
            LauncherModel.deleteCachedIcons(this)
            LauncherAppState.forceModelReload()
        }
    }
}