package com.teslacoilsw.launcher.preferences

import android.content.ComponentName
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.LightingColorFilter
import android.graphics.Paint
import android.graphics.Typeface
import android.os.Build
import android.os.Bundle
import android.preference.PreferenceManager
import android.support.v4.content.ContextCompat
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.CheckedTextView
import android.widget.CompoundButton
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.SeekBar
import android.widget.Spinner
import android.widget.TextView
import com.android.colorpicker.ColorPickerDialog
import com.android.launcher3.BubbleTextView
import com.android.launcher3.DeviceProfile
import com.android.launcher3.FastBitmapDrawable
import com.android.launcher3.LauncherAppState
import com.android.launcher3.LauncherModel
import com.android.launcher3.Utilities
import com.teslacoilsw.kotlin.dp
import com.teslacoilsw.kotlin.get
import com.teslacoilsw.kotlin.size
import com.teslacoilsw.kotlin.sp
import com.teslacoilsw.launcher.CellIconSizeSpecs
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.dock.DockDrawable
import com.teslacoilsw.launcher.icontheme.IconThemeEmpty
import com.teslacoilsw.launcher.icontheme.IconThemeLollipop
import com.teslacoilsw.launcher.widget.DisabledSegmentSeekBar
import com.teslacoilsw.launcher.widget.TypefaceHelper
import com.teslacoilsw.licensing.ProKeyGoogleLicenseChecker
import com.teslacoilsw.matcompat.TintableSwitchCompat
import com.teslacoilsw.shared.colorpicker.ColorPickerButton
import com.teslacoilsw.shared.colorpicker.ColorUtil
import com.teslacoilsw.shared.poisonlollipop.activity.PoisonActionBarActivity
import com.teslacoilsw.shared.util.MathUtils
import com.teslacoilsw.tesladirect.MarketHandler
import kotterknife.bindView
import java.text.NumberFormat

const val DOCK_SMALL_ICON_SCALE = 80
class IconLayoutSettingsActivity : PoisonActionBarActivity(), DisabledSegmentSeekBar.OnSeekBarChangeListener, CompoundButton.OnCheckedChangeListener {
    private val previewIcons: ViewGroup by bindView(R.id.preview_icons)
    private val iconScale: DisabledSegmentSeekBar by bindView(R.id.scale_seekbar)
    private val iconScaleText: TextView by bindView(R.id.scale_textview)
    private val hint: TextView by bindView(R.id.hint)
    private val labelVisible: TintableSwitchCompat by bindView(R.id.label_visible)
    private val labelFamily:Spinner by bindView(R.id.label_family)
    private val labelColor: ColorPickerButton by bindView(R.id.label_color)
    private val labelColorFrame: ViewGroup by bindView(R.id.label_color_frame)
    private val labelColorText: CheckedTextView by bindView(R.id.label_color_text)
    private val labelShadowFrame: ViewGroup by bindView(R.id.label_shadow_frame)
    private val labelShadow: CheckedTextView by bindView(R.id.label_shadow)
    private val hintButton: Button by bindView(R.id.hint_button)
    private val smallIconsSwitch: TintableSwitchCompat by bindView(R.id.scale_switch)
    private val labelSingleLineFrame: ViewGroup by bindView(R.id.label_single_line_frame)
    private val labelSingleLine: CheckedTextView by bindView(R.id.label_single_line)
    private val labelSize: SeekBar by bindView(R.id.label_size_seekbar)
    private val labelFamilyHint: TextView by bindView(R.id.label_family_hint)
    private val labelSizeHint: TextView by bindView(R.id.label_size_hint)
    private val actionBarBack: ImageView by bindView(R.id.back)

    private val novaSettingsIcon: BubbleTextView by bindView(R.id.preview_icon_nova_settings)

    private val displayMetrics by lazy { resources.displayMetrics }
    private val cellSpecs by lazy { CellIconSizeSpecs().apply { iconDrawablePaddingPx = displayMetrics.dp(4) } }

    private val stepIncrement = 5
    private val minScale = 70
    private var labelSizeDp = 10f
    private var labelFamilyValue = ""

    private lateinit var prefix: String
    private val prefs by lazy { PreferenceManager.getDefaultSharedPreferences(this) }
    private val percentFormatter by lazy {
        NumberFormat.getInstance().apply { maximumFractionDigits = 0 } }

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        overridePendingTransition(0, 0)

        if (intent == null || !intent.hasExtra("category")) {
            finish()
            return
        }

        LauncherAppState.getInstance().initDynamicGrid(this, windowManager.defaultDisplay)
        setContentView(R.layout.activity_iconlayout)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Utilities.setStatusAndNavTransparent(window, true, false, false)
            window.statusBarColor = 0x3d000000
        }

        actionBarBack.setOnClickListener({
            onBackPressed()
        })

        val dp16 = displayMetrics.dp(16)
        prefix = intent.getStringExtra("category")
        if ("drawer" == prefix) {
            val baseBgColor: Int
            if (Pref2.get().drawer.drawer_cards.get()!!) {
                baseBgColor = Pref2._Drawer.DRAWER_BG_FOR_CARDS
            } else {
                baseBgColor = Pref2.get().drawer.drawer_background_color.get()!!
            }

            findViewById<View>(R.id.fragment_container).setBackgroundColor(baseBgColor)
            if (Pref2.get().drawer.drawer_cards.get()!!) {
                val backgroundColor = ColorUtil.argb(0xff, Pref2.get().drawer.drawer_card_color.get()!!)
                val cf = LightingColorFilter(backgroundColor, Color.rgb(0, 0, 0))
                val d = ContextCompat.getDrawable(this, R.drawable.quantum_panel)!!
                d.colorFilter = cf
                d.alpha = Color.alpha(Pref2.get().drawer.drawer_card_color.get()!!)
                previewIcons.background = d
                previewIcons.setPadding(dp16, 0, dp16, 0)
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    window.statusBarColor = ColorUtil.compositeColors(0x3d000000, Pref2.get().drawer.drawerBackground)
                }
            } else {
                val backgroundColor = ColorUtil.compositeColors(Pref2.get().drawer.drawerBackground, 0xFF424242.toInt())
                val lightBg = ColorUtil.calculateContrast(Color.WHITE, backgroundColor) < 1.5
                if (lightBg)
                    actionBarBack.setColorFilter(Color.BLACK)
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    window.statusBarColor = ColorUtil.compositeColors(0x3d000000, Pref2.get().drawer.drawerBackground)
                }
            }
        } else if ("folder" == prefix) {
            val backgroundColor = ColorUtil.argb(0xff, Pref2.get().folder_window_color_raw.get()!!)
            val cf = LightingColorFilter(backgroundColor, Color.rgb(0, 0, 0))
            val d = ContextCompat.getDrawable(this, R.drawable.quantum_panel)!!
            d.colorFilter = cf
            d.alpha = Color.alpha(Pref2.get().folder_window_color_raw.get()!!)
            previewIcons.background = d
            previewIcons.setPadding(dp16, 0, dp16, 0)
//            val lp = previewIcons.layoutParams as ViewGroup.MarginLayoutParams
//            val dp48 = displayMetrics.dp(48)
//            lp.setMargins(dp48, displayMetrics.dp(32), dp48, displayMetrics.dp(12))
//            previewIcons.layoutParams = lp
        } else if ("dock" == prefix && Pref.Value.dock_background.enabled) {
            val mDockDrawable = DockDrawable(resources)
            Pref.Value.dock_background.setupDrawable(this, mDockDrawable, true)
            previewIcons.background = mDockDrawable
            val lp = previewIcons.layoutParams as RelativeLayout.LayoutParams
            lp.addRule(RelativeLayout.CENTER_VERTICAL, 1)
        }

        iconScale.max = (150 - minScale) / stepIncrement

        val isDesktop = "desktop" == prefix
        val isDock = "dock" == prefix
        var scale = prefs.getInt(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_ICON_SIZE), 100)
        if (!Pref.Value.nova_prime_enabled) {
            if ("dock" == prefix) {
                if (scale != DOCK_SMALL_ICON_SCALE && scale != 100) {
                    scale = 100
                }
            } else {
                scale = 100
            }
        }

        iconScale.progress = valueToProgress(scale)
        iconScaleText.text = Math.round(scale / 100.0f).toString() + "%"
        iconScale.isEnabled = Pref.Value.nova_prime_enabled
        smallIconsSwitch.isChecked = scale == DOCK_SMALL_ICON_SCALE

        if ("dock" == prefix) {
            hint.setText(R.string.changing_icon_size_requires_prime_dock)
            if (!Pref.Value.nova_prime_enabled) {
                iconScale.visibility = View.GONE
                smallIconsSwitch.visibility = View.VISIBLE
            }
        }

        hint.visibility = if (Pref.Value.nova_prime_enabled) View.INVISIBLE else View.VISIBLE
        hintButton.visibility = if (Pref.Value.nova_prime_enabled) View.GONE else View.VISIBLE

        labelVisible.isChecked = prefs.getBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_SHOW_LABELS), "dock" != prefix)
        labelFamilyValue = prefs.getString(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_FONT_FAMILY), TypefaceHelper.CONDENSED)
        labelSizeDp = prefs.getFloat(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_SIZE_SP), 13f)
        labelColor.color = prefs.getInt(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_COLOR), if (isDesktop || isDock) Color.WHITE else Color.BLACK)
        labelShadow.isChecked = prefs.getBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_SHADOW), isDesktop)
        labelSingleLine.isChecked = prefs.getBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_SINGLE_LINE), true)

        /* This fixes mTopDrawable which is not normally populated by xml */
        val cache = LauncherAppState.getInstance().iconCache
        var iconTheme = cache.iconTheme
        val defaultTheme = IconThemeLollipop(this)
        if (iconTheme is IconThemeEmpty) {
            iconTheme = defaultTheme
        }
        for (i in 0 until previewIcons.size) {
            val tv = previewIcons[i] as BubbleTextView
            var icon: Bitmap? = iconTheme.getThemedIconBitmap(this, cache, ComponentName.unflattenFromString(tv.tag as String))
            if (icon == null) {
                icon = defaultTheme.getThemedIconBitmap(this, cache, ComponentName.unflattenFromString(tv.tag as String))
            }
            val d = FastBitmapDrawable(icon)
            tv.setCompoundDrawables(null, d, null, null)
        }

        iconScale.setOnSeekBarChangeListener(this)
        findViewById<View>(R.id.label_visible_frame).setOnClickListener { labelVisible.toggle() }
        labelVisible.setOnCheckedChangeListener(this)
        labelShadowFrame.setOnClickListener {
            labelShadow.toggle()
            updateSpecs()
        }

        labelColorFrame.setOnClickListener {
            val dialog = ColorPickerDialog.newInstance(R.string.dialog_color_picker, labelColor.color, false, 0)
            dialog.setOnColorSelectedListener { color ->
                labelColor.color = color
                updateSpecs()
            }
            dialog.show(this@IconLayoutSettingsActivity)
        }

        labelSingleLineFrame.setOnClickListener {
            labelSingleLine.toggle()
            updateSpecs()
        }

        val labelFamilyValues = resources.getStringArray(R.array.pref_label_fontfamily_values)
        val adapter = object : ArrayAdapter<CharSequence>(this,
                R.layout.simple_spinner_item,
                resources.getTextArray(R.array.pref_label_fontfamily)) {
            override fun getDropDownView(position: Int, convertView: View?, parent: ViewGroup): View {
                val view = super.getDropDownView(position, convertView, parent) as TextView
                view.typeface = TypefaceHelper.create(context, labelFamilyValues[position], Typeface.NORMAL)
                return view
            }

            override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
                val view = super.getView(position, convertView, parent) as TextView
                view.typeface = TypefaceHelper.create(context, labelFamilyValues[position], Typeface.NORMAL)
                return view
            }
        }
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        labelFamily.adapter = adapter
        for (i in labelFamilyValues.indices) {
            val v = labelFamilyValues[i]
            if (labelFamilyValue == v) {
                labelFamily.setSelection(i)
                break
            }
        }

        labelFamily.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View?, position: Int, id: Long) {
                val value = labelFamilyValues[position]
                labelFamilyValue = value
                updateSpecs()
            }

            override fun onNothingSelected(parent: AdapterView<*>) {

            }
        }

        val progress: Int
        if (labelSizeDp <= 11)
            progress = 0
        else if (labelSizeDp <= 12)
            progress = 1
        else if (labelSizeDp <= 13)
            progress = 2
        else if (labelSizeDp <= 14.4f)
            progress = 3
        else if (labelSizeDp <= 16)
            progress = 4
        else if (labelSizeDp <= 18)
            progress = 5
        else if (labelSizeDp <= 21)
            progress = 6
        else
            progress = 2
        labelSize.progress = progress
        labelSize.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar, progress: Int, fromUser: Boolean) {
                when (progress) {
                    0 -> labelSizeDp = 11f
                    1 -> labelSizeDp = 12f
                    2 -> labelSizeDp = 13f
                    3 -> labelSizeDp = 14.4f
                    4 -> labelSizeDp = 16f
                    5 -> labelSizeDp = 18f
                    6 -> labelSizeDp = 21f
                }
                updateSpecs()
            }

            override fun onStartTrackingTouch(seekBar: SeekBar) {

            }

            override fun onStopTrackingTouch(seekBar: SeekBar) {

            }
        })


        onCheckedChanged(labelVisible, labelVisible.isChecked)

        hintButton.setOnClickListener({
            val intent = MarketHandler.getMarketLink(ProKeyGoogleLicenseChecker.PACKAGE)
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            startActivity(intent)
        })

        smallIconsSwitch.setOnClickListener({
            iconScale.progress = if (smallIconsSwitch.isChecked) valueToProgress(80)
                                 else valueToProgress(100)
        })

    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
        }

        return super.onOptionsItemSelected(item)
    }

    private val mTextPaint = Paint()
    private fun updateLayoutForDockBackground() {
        val drawablePadding = resources.getDimensionPixelSize(R.dimen.dynamic_grid_icon_drawable_padding)
        val edgeMarginPx = resources.getDimensionPixelSize(R.dimen.dynamic_grid_edge_margin)
        val textPaint = mTextPaint
        textPaint.textSize = cellSpecs.iconTextSizePx.toFloat()
        val fm = textPaint.fontMetrics
        val textSize = Math.ceil((if (cellSpecs.iconTextSingleLine) 1f else DeviceProfile.TWO_LINE_MULTIPLER) * Math.ceil((fm.bottom - fm.top).toDouble())).toInt()
        val hotseatCellHeightPx = cellSpecs.iconSizePx + if (cellSpecs.iconTextVisible) drawablePadding + textSize else 0
        val hotseatBarHeightPx = hotseatCellHeightPx + 4 * edgeMarginPx

        previewIcons.layoutParams.height = hotseatBarHeightPx
    }

    private fun updateSpecs() {
        val baseSize = resources.getDimensionPixelSize(R.dimen.app_icon_size)

        cellSpecs.iconTextVisible = labelVisible.isChecked
        cellSpecs.iconDrawablePaddingPx = if (cellSpecs.iconTextVisible) displayMetrics.dp(4) else 0
        cellSpecs.iconTextColor = labelColor.color
        cellSpecs.iconTextShadow = labelShadow.isChecked
        cellSpecs.iconFontFamily = labelFamilyValue
        cellSpecs.iconTextSingleLine = labelSingleLine.isChecked

        if (cellSpecs.iconTextSingleLine) {

            novaSettingsIcon.text = "Nova Settings"
        } else {

            novaSettingsIcon.text = "Nova\nSettings"
        }

        cellSpecs.iconSizePx = baseSize
        cellSpecs.iconTextSizeSp = if (cellSpecs.iconTextVisible) labelSizeDp else 0f
        cellSpecs.iconTextSizePx = if (cellSpecs.iconTextVisible) displayMetrics.sp(labelSizeDp) else 0

        val maxScale = LauncherAppState.getInstance().dynamicGrid.deviceProfile.getMaxIconScale(this, prefix, cellSpecs)

        val scale = Math.min(maxScale, progressToValue(iconScale.progress) / 100f)
        cellSpecs.iconSizePx = MathUtils.roundInt(baseSize * scale)

        val maxScaleInt = Math.min(150, MathUtils.ceilInt(maxScale * 100))

        val disableAfter = MathUtils.ceilInt((maxScaleInt - minScale) / stepIncrement.toFloat())
        iconScale.setDisableAfter(disableAfter)
        iconScaleText.text = progressToValue(iconScale.progress).toString() + "%"

        if (Pref.Value.nova_prime_enabled) {
            if (disableAfter < iconScale.max && iconScale.progress >= disableAfter) {
                hint.setText(R.string.icon_layout_further_increase)
                hint.visibility = View.VISIBLE
            } else {
                hint.visibility = View.INVISIBLE
            }
        }

        for (i in 0 until previewIcons.childCount) {
            val tv = previewIcons.getChildAt(i) as BubbleTextView
            tv.applyCellSpecs(cellSpecs)
        }

        if ("dock" == prefix && Pref.Value.dock_background.enabled) {
            updateLayoutForDockBackground()
        }

    }

    private fun progressToValue(progress: Int): Int {
        val value = progress * stepIncrement + minScale
        return value
    }

    private fun valueToProgress(value: Int): Int {
        val progress = (value - minScale) / stepIncrement
        return progress
    }

    override fun onProgressChanged(seekBar: DisabledSegmentSeekBar, progress: Int, fromUser: Boolean) {
        updateSpecs()
    }

    override fun onStartTrackingTouch(seekBar: DisabledSegmentSeekBar) {

    }

    override fun onStopTrackingTouch(seekBar: DisabledSegmentSeekBar) {

    }

    override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {
        labelFamily.isEnabled = isChecked
        labelSize.isEnabled = isChecked
        labelColor.isEnabled = isChecked
        labelColorText.isEnabled = isChecked
        labelColorFrame.isEnabled = isChecked
        labelShadow.isEnabled = isChecked
        labelShadowFrame.isEnabled = isChecked
        labelSingleLine.isEnabled = isChecked
        labelSingleLineFrame.isEnabled = isChecked
        labelSizeHint.isEnabled = isChecked
        labelFamilyHint.isEnabled = isChecked

        updateSpecs()
    }

    public override fun onPause() {
        super.onPause()

        overridePendingTransition(0, 0)

        prefs.edit()
            .putInt(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_ICON_SIZE), progressToValue(iconScale.progress))
            .putBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_SHOW_LABELS), labelVisible.isChecked)
            .putString(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_FONT_FAMILY), labelFamilyValue)
            .putInt(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_COLOR), labelColor.color)
            .putBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_SHADOW), labelShadow.isChecked)
            .putFloat(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_SIZE_SP), labelSizeDp)
            .putBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_SINGLE_LINE), labelSingleLine.isChecked)
            .apply()

        if (prefix == "desktop" || prefix == "drawer") {
            LauncherModel.invalidateCachedIcons(this)
            LauncherAppState.getInstance().model.resetLoadedState(true, true)
        }

        NovaLauncher.postRecreate()
    }
}
