package com.teslacoilsw.launcher.preferences.fragments

import android.os.Bundle
import android.preference.Preference
import android.preference.PreferenceCategory
import com.teslacoilsw.launcher.R

class IntegrationPreferences : NovaPreferenceFragment() {
    private val integrations = ArrayList<IntegrationPrefHelper>();

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        integrations.clear()
        addPreferencesFromResource(R.xml.preferences_integrations)

        val googleScreen = findPreference("google_now") as PreferenceCategory
        integrations.add(GoogleNowPreferences(this, googleScreen))

        val sesameScreen = findPreference("sesame") as PreferenceCategory
        integrations.add(SesamePreferences(this, sesameScreen))

        for (p in integrations)
            p.onCreate()
    }

    override fun onResume() {
        super.onResume()
        for (p in integrations) {
            val shouldShow = p.onResumeShouldShow()
            if (!shouldShow) {
                preferenceScreen.removePreference(p.preferenceCategory)
            } else {
                preferenceScreen.addPreference(p.preferenceCategory)
            }
        }
    }
}

abstract class IntegrationPrefHelper(val parent: IntegrationPreferences, val preferenceCategory: PreferenceCategory) {
    protected val packageManager by lazy(LazyThreadSafetyMode.NONE) { parent.activity.applicationContext.packageManager!! }
    abstract fun onCreate()
    abstract fun onResumeShouldShow(): Boolean

    fun findPreference(key: String): Preference? = preferenceCategory.findPreference(key)

}
