package com.teslacoilsw.launcher.preferences.fragments

import android.content.ActivityNotFoundException
import android.content.Intent
import android.net.Uri
import android.preference.Preference
import android.preference.PreferenceCategory
import android.text.Html
import android.widget.Toast
import com.teslacoilsw.launcher.BuildConfig
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.util.PackageNames
import com.teslacoilsw.tesladirect.MarketHandler

class SesamePreferences(parent: IntegrationPreferences, preferenceScreen: PreferenceCategory) : IntegrationPrefHelper(parent, preferenceScreen) {
    val warning by lazy { preferenceScreen.findPreference("sesame_permission")!! }
    var warningIsRemoved = false
    override fun onCreate() {
    }

    override fun onResumeShouldShow(): Boolean {

        if (checkSesameEnabled()) {
            preferenceCategory.removePreference(warning)
            warningIsRemoved = true
        } else {
            if (warningIsRemoved) {
                preferenceCategory.addPreference(warning)
                warningIsRemoved = false
            }
            warning.setOnPreferenceClickListener {
                try {
                    val intent = Intent("ninja.sesame.app.action.ENABLE_INTEGRATION")
                    intent.setPackage(PackageNames.SESAME)
                    intent.putExtra("package", BuildConfig.APPLICATION_ID)
                    parent.startActivityForResult(intent, 1)
                } catch (e: ActivityNotFoundException) {
                    Toast.makeText(parent.activity, R.string.update_to_continue, Toast.LENGTH_SHORT).show()
                    val intent = MarketHandler.getMarketLink(MarketHandler.GOOGLE, PackageNames.SESAME)
                    intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP or Intent.FLAG_ACTIVITY_NEW_TASK)
                    parent.startActivity(intent)
                }
                true
            }

            findPreference(Pref2.get().sesame_app_search.key())!!.isEnabled = false
            findPreference(Pref2.get().sesame_shortcut_menu.key())!!.isEnabled = false
        }

        val sesame_app = findPreference("sesame_app")!!
        sesame_app.onPreferenceClickListener = Preference.OnPreferenceClickListener {
            val intent = packageManager.getLaunchIntentForPackage(PackageNames.SESAME)
            try {
                parent.activity.startActivity(intent)
            } catch (e: Exception) {
                e.printStackTrace()
                Toast.makeText(parent.activity, parent.getString(R.string.activity_not_found), Toast.LENGTH_SHORT).show()
            }

            true
        }

        val (appName, versionName, versionCode) = try {
            val pi = packageManager.getPackageInfo(PackageNames.SESAME, 0)
            Triple(pi.applicationInfo.loadLabel(packageManager), pi.versionName, pi.versionCode)
        } catch (e: Exception) {
            Triple("", "", 0)
        }
        if (versionCode > 0) {
            val sb = StringBuilder()
            sb.append(appName).append(' ')
            sb.append("<small>")
            sb.append(versionName)
            sb.append("</small>")
            sesame_app.title = Html.fromHtml(sb.toString())
        }

        var hasSesamePermission = false

        if (hasSesamePermission) {

        }

        return true
    }

    fun checkSesameEnabled() : Boolean {
        val c = try {
            parent.activity.contentResolver.query(Uri.parse("content://ninja.sesame.app.provider.config/integration?package=com.teslacoilsw.launcher"), null, null, null, null)
        } catch (e: Exception ) {
            null
        }

        if (c != null) {
            try {
                if (c.moveToFirst()) {
                    val enabledIndex = c.getColumnIndexOrThrow("enabled")
                    if (c.getString(enabledIndex) == "true")
                        return true
                }
            } catch (e: IllegalArgumentException) {
                e.printStackTrace()
            } finally {
                try { c.close() } catch (e: Exception) { e.printStackTrace() }
            }
        }
        return false
    }

}