package com.teslacoilsw.launcher.quicksearchbar

import android.content.Context
import android.graphics.Point
import android.graphics.Rect
import android.support.v4.view.ViewCompat
import android.support.v7.widget.LinearLayoutCompat
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import com.android.launcher3.CheckLongPressHelper
import com.android.launcher3.DragLayer
import com.android.launcher3.Launcher
import com.f2prateek.rx.preferences.Preference
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.daynight.DayNightManager
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.util.DpUtil
import com.teslacoilsw.launcher.util.SearchUtils
import rx.Observable
import rx.Subscription
import rx.subscriptions.CompositeSubscription

/**
 * Created by kevin on 3/14/16.
 */
private val ORIENTATION_ATTRS = intArrayOf(android.R.attr.orientation)
class QsbFrameLayout @JvmOverloads constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int = 0) : FrameLayout(context, attrs, defStyleAttr), DragLayer.TouchCompleteListener {
    private var subscription: Subscription? = CompositeSubscription()
    @LinearLayoutCompat.OrientationMode val orientation: Int
    var launcher: Launcher? = null

    private var stylePreference: Preference<SearchBarStyle>? = null

    private val searchIcon by lazy(LazyThreadSafetyMode.NONE) { findViewById<ImageView>(R.id.qsb_base_search_button) }
    private val searchSpace by lazy(LazyThreadSafetyMode.NONE) { findViewById<View>(R.id.qsb_base_search_space) }
    private val voiceIcon by lazy(LazyThreadSafetyMode.NONE) { findViewById<ImageView>(R.id.qsb_base_voice_button) }
    private val appDrawerIcon by lazy(LazyThreadSafetyMode.NONE) { findViewById<ImageView>(R.id.qsb_base_app_drawer) }

    private val mLongPressHelper = CheckLongPressHelper(this)


    init {
        val a = context.obtainStyledAttributes(attrs, ORIENTATION_ATTRS)
        orientation = a.getInt(0, LinearLayout.HORIZONTAL)
        a.recycle()
    }

    override fun onInterceptTouchEvent(ev: MotionEvent): Boolean {
        if (ev.action == MotionEvent.ACTION_DOWN) {
            mLongPressHelper.cancelLongPress()
            launcher?.dragLayer?.setTouchCompleteListener(this)
        }

        // Consume any touch events for ourselves after longpress is triggered
        if (mLongPressHelper.hasPerformedLongPress()) {
            mLongPressHelper.cancelLongPress()
            return true
        }

        mLongPressHelper.onTouchEvent(ev)

        return super.onInterceptTouchEvent(ev)
    }


    override fun onTouchEvent(ev: MotionEvent): Boolean {
        // If the widget does not handle touch, then cancel
        // long press when we release the touch
        mLongPressHelper.onTouchEvent(ev)
        return false
    }


    override fun cancelLongPress() {
        super.cancelLongPress()
        mLongPressHelper.cancelLongPress()
    }


    override fun onTouchComplete() {
        if (!mLongPressHelper.hasPerformedLongPress()) {
            // If a long press has been performed, we don't want to clear the record of that since
            // we still may be receiving a touch up which we want to intercept
            mLongPressHelper.cancelLongPress()
        }
    }


    override fun isLongClickable(): Boolean {
        /* This prevents setPressed from being called by the parent view */
        return true
    }


    override fun setOnLongClickListener(l: View.OnLongClickListener?) {
        mLongPressHelper.setOnLongClickListener(l)
    }

    fun searchClick(v: View) {

        val _background = background

        val sourceBounds: Rect
        val framePos: QsbFramePosition?
        /* Currently the fancy searchbar animation requires google now integration,
         * Don't bother even trying without as we'd have to wait for the failure state
         */
        if (launcher?.hasGoogleNow() == false || stylePreference?.get()?.canUsePillAnimation() == false) {
            framePos = null

            val pos = IntArray(2)
            v.getLocationOnScreen(pos)
            sourceBounds = Rect(pos[0], pos[1], pos[0] + v.measuredWidth, pos[1] + v.measuredHeight)
        } else if (_background != null) {
            val pos = IntArray(2)
            getLocationOnScreen(pos)
            val bounds = Rect(pos[0], pos[1], pos[0] + measuredWidth, pos[1] + measuredHeight)

            /* Not sure if we can get this from the drawable. Maybe with `opticalBounds` but that
             * is @hide anyway */
            val widthPadding = DpUtil.getPixelSize(5)
            val heightPadding = DpUtil.getPixelSize(4)

            bounds.left += widthPadding
            bounds.top += heightPadding
            bounds.right -= widthPadding
            bounds.bottom -= heightPadding

            searchIcon.getLocationOnScreen(pos)
            pos[0] += searchIcon.measuredWidth/2 - bounds.left
            pos[1] += searchIcon.measuredHeight/2 - bounds.top

            val bgStyle = stylePreference?.get()?.bgStyle ?: SearchBarStyle.BG_ROUNDRECT
            val round = bgStyle == SearchBarStyle.BG_PILLRECT || bgStyle == SearchBarStyle.BG_PILL
            framePos = QsbFramePosition(bounds, Point(pos[0], pos[1]), round, round)
            sourceBounds = bounds
        } else {
            val pos = IntArray(2)
            searchIcon.getLocationOnScreen(pos)
            val bounds = Rect(pos[0], pos[1], pos[0] + searchIcon.measuredWidth, pos[1] + searchIcon.measuredHeight)

            /* Not sure if we can get this from the drawable. Maybe with `opticalBounds` but that
             * is @hide anyway */
            if (searchIcon.background != null) {
                val widthPadding = DpUtil.getPixelSize(6)
                val heightPadding = DpUtil.getPixelSize(4)
                bounds.left += widthPadding
                bounds.top += heightPadding
                bounds.right -= widthPadding
                bounds.bottom -= heightPadding
            }

            val minSize = DpUtil.getPixelSize(48)
            if (bounds.width() < minSize) {
                val diff = minSize - bounds.width()
                bounds.left -= diff/2
                bounds.right += diff - diff/2
            }

            if (bounds.height() > bounds.width()) {
                /* This looks bonkers */
                val diff = bounds.height() - bounds.width()
                bounds.top += diff/2
                bounds.bottom -= diff - diff/2
            }

            pos[0] = bounds.width()/2
            pos[1] = bounds.height()/2

            framePos = QsbFramePosition(bounds, Point(pos[0], pos[1]), true, roundRight = true)
            sourceBounds = bounds
        }

        SearchUtils.startGlobalSearch(context, null, false, null, sourceBounds, null, framePos)
    }

    override fun onFinishInflate() {
        super.onFinishInflate()
        searchIcon.setOnClickListener(this::searchClick)
        searchSpace.setOnClickListener(this::searchClick)
        voiceIcon.setOnClickListener { v -> launcher?.onClickVoiceButton(v) }
        appDrawerIcon.setOnClickListener { v -> launcher?.onClickAllAppsButton(v) }
    }

    fun setPreference(stylePreference: Preference<SearchBarStyle>) {
        subscription?.unsubscribe()
        subscription = null

        this.stylePreference = stylePreference
        if (ViewCompat.isAttachedToWindow(this))
            subscribe()
    }

    private fun subscribe() {
        stylePreference?.let {
            subscription = Observable.combineLatest(it.asObservable(),
                    DayNightManager.get().asObservable(),
                    Pref2.get().night_mode_auto_searchbar.asObservable()) {
                    searchBarStyle, searchbar_night_mode, is_night_mode ->
                    if (is_night_mode!! && searchbar_night_mode!!)
                        searchBarStyle.nightModeStyle
                    else
                        searchBarStyle
            }.subscribe { searchBarStyle ->
                searchBarStyle.apply(this@QsbFrameLayout)
                updatePosition()
            }
        }
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        if (changed)
            updatePosition()
    }

    /**
     * When the widget has been moved/resized update to show/hide content
     * We can't use our own onLayout as then we won't know about being moved within the CellLayout
     */
    fun updatePosition() {
        parent.let {
            if (it is View) {
                stylePreference?.get()?.layout(this@QsbFrameLayout, it.left, it.top, it.right, it.bottom)
            }
        }
    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        subscribe()
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        subscription?.unsubscribe()
        subscription = null
    }

    override fun hasOverlappingRendering(): Boolean {
        return false
    }

    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        if (alpha < 0.1f)
            return false
        return super.dispatchTouchEvent(ev)
    }
}

data class QsbFramePosition(val frameRect: Rect, val logoOffset: Point, val roundLeft: Boolean, val roundRight: Boolean)
