package com.teslacoilsw.launcher.util.picasso

import android.graphics.Bitmap
import android.graphics.BitmapShader
import android.graphics.Canvas
import android.graphics.Matrix
import android.graphics.Paint
import android.graphics.Shader
import com.squareup.picasso.Transformation


class CropCircleTransformation(val scale: Float = 1.0f) : Transformation {

    val canvas = object: ThreadLocal<Canvas>() {
        override fun initialValue(): Canvas {
            return Canvas()
        }
    }

    val paint = object: ThreadLocal<Paint>() {
        override fun initialValue(): Paint {
            return Paint()
        }
    }
    val matrix = object: ThreadLocal<Matrix>() {
        override fun initialValue(): Matrix {
            return Matrix()
        }
    }

    override fun transform(source: Bitmap): Bitmap {
        val size = Math.min(source.width, source.height)

        val width = (source.width - size) / 2
        val height = (source.height - size) / 2

        val bitmap = Bitmap.createBitmap(size, size, Bitmap.Config.ARGB_8888)

        val canvas = canvas.get()
        canvas.setBitmap(bitmap)
        val paint = paint.get()
        val shader = BitmapShader(source, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP)
        if (width != 0 || height != 0) {
            // source isn't square, move viewport to center
            val matrix = matrix.get()
            matrix.setTranslate(-width.toFloat(), -height.toFloat())
            shader.setLocalMatrix(matrix)
        }
        paint.shader = shader
        paint.isAntiAlias = true

        val center = size/2f
        val radius = center * scale;
        canvas.drawCircle(center, center, radius, paint)
        canvas.setBitmap(null)
        source.recycle()

        return bitmap
    }

    override fun key(): String {
        return "CropCircleTransform"
    }
}