package com.teslacoilsw.launcher.util.picasso

import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.PorterDuff
import android.graphics.PorterDuffColorFilter
import android.graphics.drawable.Drawable
import com.squareup.picasso.Transformation
import com.teslacoilsw.launcher.bitmaputils.ThreadLocalCanvas
import com.teslacoilsw.launcher.bitmaputils.ThreadLocalPaint
import com.teslacoilsw.launcher.bitmaputils.ThreadLocalRect

class PlateTransformation(private val bgColor: Int, private val fgColor: Int) : Transformation {

    private val fgColorFilter = PorterDuffColorFilter(fgColor, PorterDuff.Mode.SRC_ATOP)

    override fun key(): String = toString()

    override fun transform(source: Bitmap): Bitmap = transform(source, true);

    fun transform(source: Bitmap, doRecycle: Boolean): Bitmap {
        val out = Bitmap.createBitmap(source.width, source.height, Bitmap.Config.ARGB_8888)
        val p = paint.get()
        canvas.withBitmap(out) {
            p.color = bgColor
            drawCircle(out.width/2f, out.height/2f, out.width/2f, p)
            p.color = Color.WHITE

            p.colorFilter = fgColorFilter

            val s = srcRect.get(0, 0, source.width, source.height)
            val inset = source.width/6
            val d = dstRect.get(inset, inset, source.width - inset, source.height - inset)
            drawBitmap(source, s, d, p)
            p.colorFilter = null
        }
        if (doRecycle)
            source.recycle()
        return out
    }

    fun transform(source: Drawable, size: Int): Bitmap {
        val out = Bitmap.createBitmap(size, size, Bitmap.Config.ARGB_8888)
        val p = paint.get()
        canvas.withBitmap(out) {
            p.color = bgColor
            drawCircle(out.width/2f, out.height/2f, out.width/2f, p)
            p.color = Color.WHITE

            val oldBounds = dstRect.get(source.bounds)
            val inset = size/6
            source.setBounds(inset, inset, size - inset, size - inset)
            source.colorFilter = fgColorFilter
            source.draw(this)
            source.colorFilter = null
            source.bounds = oldBounds
        }
        return out
    }



    override fun toString(): String {
        return "PlateTransformation(${Integer.toHexString(bgColor)}, ${Integer.toHexString(fgColor)})"
    }


    companion object {
        private val canvas = ThreadLocalCanvas()
        private val paint = ThreadLocalPaint()
        private val srcRect = ThreadLocalRect()
        private val dstRect = ThreadLocalRect()
    }
}