package com.teslacoilsw.notifier.widget

import android.content.Context
import android.graphics.drawable.Drawable
import android.os.Build
import android.os.Handler
import android.os.Message
import android.service.notification.StatusBarNotification

/**
 * Follows the API of V23 Icon but backwards compatible-ish
 */

interface IconCompat {

    fun loadDrawableAsync(context: Context, andThen: Message)

    fun loadDrawableAsync(context: Context, listener: OnDrawableLoadedListener,
                          handler: Handler)

    fun loadDrawable(context: Context): Drawable


    interface OnDrawableLoadedListener {
        fun onDrawableLoaded(d: Drawable)
    }
}

fun getSmallIcon(sbn: StatusBarNotification): IconCompat? {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
        sbn.notification.smallIcon?.let { return IconCompatV23(it) }
    } else {
        if (sbn.notification.icon != 0)
            return IconCompatResource.create(sbn.packageName, sbn.notification.icon)
    }
    return null
}

fun getLargeIcon(sbn: StatusBarNotification): IconCompat? {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
        sbn.notification.getLargeIcon()?.let { return IconCompatV23(it) }
    } else {
        sbn.notification.largeIcon?.let { return IconCompatDrawable.create(it) }
    }
    return null
}
